#include "matrix.h"
#include <iostream>

Matrix::Matrix(const unsigned int dimension)
  : value_((dimension==0)?0:(new double* [dimension])),
    dimension_(dimension)
{
  for (unsigned int i=0; i < dimension_; ++i) {
    value_[i] = new double[dimension_];
  }

  *this = 0.0;
}

Matrix::Matrix(const double *diag, unsigned int dim)
  : value_(0), dimension_(0)
{
  this->diag(diag, dim);
}

Matrix::Matrix(const Matrix& clone)
  : value_((clone.dimension_==0)?0:(new double* [clone.dimension_])),
    dimension_(clone.dimension_)
{
  for (unsigned int i=0; i < dimension_; ++i) {
    value_[i] = new double[dimension_];
  }

  *this = clone;
}

Matrix::~Matrix()
{
  if (this->value_ != 0) {
    for (unsigned int i=0; i < dimension_; ++i) {
      if (this->value_[i] != NULL) delete [] this->value_[i];
    }
    delete [] this->value_;
    this->value_ = 0;
  }
}

void Matrix::changeDimension(const unsigned int dimension_)
{
  /* delete the old matrix */
  if (this->value_ != 0) {
    for (unsigned int i=0; i < this->dimension_; ++i) {
      delete [] this->value_[i];
    }
    delete [] this->value_;
    this->value_ = 0;
  }

  this->dimension_ = dimension_;

  if (dimension_ != 0) {
    this->value_ = new double* [dimension_];
    for (unsigned int i=0; i < dimension_; ++i) {
      value_[i] = new double[dimension_];
    }
  }

  *this = 0.0;
}

void Matrix::add(const Matrix *mat)
{
  for (unsigned int i=0; i < this->dimension_; ++i) {
    for (unsigned int j=0; j < this->dimension_; ++j) {
      this->value_[i][j] = this->value_[i][j] + mat->value_[i][j];
    }
  }
}

void Matrix::subtract(const Matrix *mat)
{
  for (unsigned int i=0; i < this->dimension_; ++i) {
    for (unsigned int j=0; j < this->dimension_; ++j) {
      this->value_[i][j] = this->value_[i][j] - mat->value_[i][j];
    }
  }
}

auto_array<double> Matrix::band(int& n, int& nsdiag) const
{
  if (this->dimension_ == 0) {
    n = 0;
    nsdiag = 0;
    return auto_array<double>(NULL);
  }

  n = this->dimension_;
  nsdiag = 0;

  // check how many non-zero diagonals exist
  for (unsigned int i=0; i < this->dimension_; ++i) {
    for (unsigned int j=0; i+j < this->dimension_; ++j) {
      if (this->value_[i+j][j] != 0.0) {
        nsdiag=i;
        break;
      }
    }
  }

  // copy and transform the matrix into band-format
  auto_array<double> bandValue (new double [n*(nsdiag+1)]);
  int valcounter=0;
  for (unsigned int i=0; i<this->dimension_; ++i) {
    for (int j=0; j <= nsdiag; ++j) {
      if ((j+i) < this->dimension_)
        bandValue.get()[valcounter++] = this->value_[j+i][i];
      else
        bandValue.get()[valcounter++] = 0.0;
    }
  }

  return bandValue;
}

Matrix& Matrix::inject(const Matrix& clone, unsigned int index)
{
  for (unsigned int this_i=index, clone_i=0;
       (this_i < this->dimension_) && (clone_i < clone.dimension_);
       ++this_i, ++clone_i) {
    for (unsigned int this_j=index, clone_j=0;
         (this_j < this->dimension_) && (clone_j < clone.dimension_);
         ++this_j, ++clone_j) {
      this->value_[this_i][this_j] = clone.value_[clone_i][clone_j];
    }
  }

  return *this;
}

void Matrix::transform(double* vec) const
{
  if (vec == NULL) return;

  double* temp = new double [this->dimension_];
  for (unsigned int i=0; i<this->dimension_; ++i) {
    temp[i] = vec[i];
    vec[i] = 0.0;
  }

  for (unsigned int i=0; i<this->dimension_; ++i) {
    for (unsigned int j=0; j<this->dimension_; ++j) {
      vec[i] += temp[j]*value_[i][j];
    }
  }

  delete [] temp;
}

void Matrix::diag(const double *diag, unsigned int dim)
{
  if (dim != this->dimension_)
    this->changeDimension(dim);

  for (unsigned int i=0; i < dimension_; ++i)
    this->value_[i][i] = diag[i];
}

Matrix& Matrix::operator=(const Matrix& operand)
{
  // check for self-assignment
  if (this == &operand) return *this;

  // resize the matrix if necessary
  if (operand.dimension_ != this->dimension_)
    this->changeDimension(operand.dimension_);

  // copy the values
  for (unsigned int i=0; i < dimension_; ++i) {
    for (unsigned int j=0; j < dimension_; ++j) {
      this->value_[i][j] = operand.value_[i][j];
    }
  }

  return *this;
}

Matrix& Matrix::operator=(double value_)
{
  for (unsigned int i=0; i < dimension_; ++i) {
    for (unsigned int j=0; j < dimension_; ++j) {
      this->value_[i][j] = value_;
    }
  }

  return *this;
}

Matrix& Matrix::operator+=(const Matrix& operand)
{
  this->add(&operand);
  return *this;
}

Matrix& Matrix::operator-=(const Matrix& operand)
{
  this->subtract(&operand);
  return *this;
}


int Matrix::split(Matrix& a, Matrix& b, std::vector<unsigned int>& indexMap,
                  std::list<unsigned int>& indices_a) const
{
  unsigned int size = indices_a.size();

  if (this->dimension_ < size) {
    return -1;
  }

  a.changeDimension(size);

  b.changeDimension(this->dimension_-size);

  std::list<unsigned int> indices_b;

  indices_a.sort();

  unsigned int n;
  unsigned int aIndex;
  unsigned int lastIndex = *(indices_a.begin())+1;
  // just so lastIndex isn't by coincidence equal to the first index in the
  // list
  std::list<unsigned int>::iterator itr;
  for (itr = indices_a.begin(), n=0, aIndex=0;
       itr != indices_a.end(); itr++, n++, aIndex++) {
    if (*itr >= dimension_ || *itr == lastIndex) return -1;
    // an index is outside dimension_ or there is a double index
    while (*itr > n) {
      indices_b.push_back(n++);
      indexMap.push_back(indices_b.size()-1);
    }
    indexMap.push_back(aIndex);
    lastIndex = *itr;
  }
  for (n=n; n<dimension_; n++) {
    indices_b.push_back(n);
    indexMap.push_back(indices_b.size()-1);
  }

  unsigned int row;
  unsigned int col;
  unsigned int mat_row = 0;
  unsigned int mat_columns = 0;
  std::list<unsigned int>::iterator columns;
  for (itr = indices_a.begin(); itr != indices_a.end(); itr++) {
    for (columns = indices_a.begin(); columns != indices_a.end(); columns++) {
      row = *itr;
      col = *columns;
      a.value_[mat_row][mat_columns] = this->value_[row][col];
      mat_columns++;
    }
    mat_columns=0;
    mat_row++;
  }

  mat_row = 0;
  mat_columns = 0;
  for (itr = indices_b.begin(); itr != indices_b.end(); itr++) {
    for (columns = indices_b.begin(); columns != indices_b.end(); columns++) {
      row = *itr;
      col = *columns;
      b.value_[mat_row][mat_columns] = this->value_[row][col];
      mat_columns++;
    }
    mat_columns=0;
    mat_row++;
  }

  return 0;
}
