
#ifndef NCUTSETTING_H
#define NCUTSETTING_H 1

namespace ncut
{

  /**
   * @brief all settings needed by the algorithm
   * This class stores all parameters. It is referenced by every class.
   * If parameters are not given in the ini file default values are used.
   */
class Setting
{
  public:

    // ncut segmentation parameters
    unsigned int ncut_tree_depth; /**< the maximum depth of the dendogram */
    unsigned int number_of_eigenvectors; /**< number of eigenvectors in each Ncut
                                          * used for bi-partitioning the image */
    unsigned int number_of_eigenvector_splittest_steps; /**< number of splitting
        points in the interval
        of possible eigenvector values that are tested for their ncut quality*/

    // element information gathering parameters
    unsigned int variation_bins; /**< number of accumulator bins used for
                                  * variation measurement */
    double saturation_multiplier; /**< the saturation of each pixel in a sequence
                             * is multiplied by this value */

    // similarity calculation parameters
    double intensity_weight; /**< weight for the intensity used for the
                              * similarity calculation */
    double position_weight; /**< weight for the position used for the
                             * similarity calculation */
    double intensity_falloff; /**< falloff of the similarity along the intensity
                               * difference of two elements */
    double distance_falloff; /**< falloff of the similarity along the position
                              * difference of two elements */
    int stretch_saturation; /**< wheter strech saturation is used or not (1=on)*/

    // merge parameters
    int merge_segments; /**< wheter the segments should be merged (1=on)*/
    double merge_threshold; /**< the minimum similarty where segments are
                             * still merged */
    unsigned int minimum_segment_count; /**< the minimum number of segments,
         segments won't be merged any further if this number is reached*/
    int stretch_saturation_merge; /**< wheter the saturation should be
         streched while calculating similarty for merging (1=on)*/

    // segment splitting
    int connected_segments; /**< wheter segmentes consisting of multiple
                             * unconnected components should be split (1=yes) */

    // segment identifier lookup parameters
    unsigned int segment_adult_age; /**< the age from which on an element should
                                     * not get lost if possible*/
    double segment_age_weight; /**< the influence of the age on the overall
                                * frame window similarity */
    double min_segment_attach_similarity; /**< minimum similarity two segments of
          different frame windows must have to be considered the same segment*/

    // delayed segment visualization
    unsigned int segment_delay; /**< delay in frames after which segments are
                                 * shown in the result (to increase stability) */

    // saturation stretching function (only used if either
    // stretch_saturation or stretch_saturation_merge is set to 1)
    double stretch_point0_x; /**< leftmost point 0 in similarity streching function (x-coordinate)*/
    double stretch_point0_y; /**< leftmost point 0 in similarity streching function (y-coordinate)*/
    double stretch_point1_x; /**< point 1 in similarity streching function (x-coordinate) */
    double stretch_point1_y; /**< point 1 in similarity streching function (y-coordinate) */
    double stretch_point2_x; /**< point 2 in similarity streching function (x-coordinate) */
    double stretch_point2_y; /**< point 2 in similarity streching function (y-coordinate) */
    double stretch_point3_x; /**< rightmost point 3 in similarity streching function (x-coordinate)*/
    double stretch_point3_y; /**< rightmost point 3 in similarity streching function (y-coordinate)*/

    // visualization parameters
    int visualisation_mode; /**< visualistation mode: 0=segmented Objects 1=original image */
    int drawTrace; /**< draw old midpoint positions of final segments*/
    int drawCross; /**< draw a cross at the actual midpoint position of final segment*/
    int drawLine; /**< connect old midpoint positions of final segments */
    int tracelength; /**< length of the trace in frames */

    /**
     * @brief Initialize all values to defaults.
     */
    Setting();

    /**
     * @brief %Setting copy constructor.
     * @param clone The @c Setting to be cloned.
     */
    Setting(const Setting& clone);

    /**
     * @brief %Setting assignment operator.
     * @param clone The @c Setting to be cloned.
     * @return This object.
     */
    virtual Setting& operator=(const Setting& clone);

    /**
     * @brief %Setting destructor.
     */
    virtual ~Setting();
};

Setting::Setting()
  : // ncut segmentation paramters
    ncut_tree_depth(4),
    number_of_eigenvectors(2),
    number_of_eigenvector_splittest_steps(10),

    // element information gathering parameters
    variation_bins(100),
    saturation_multiplier(4.0),

    // similarity calculation parameters
    intensity_weight(1.0),
    position_weight(0.0),
    intensity_falloff(0.1),
    distance_falloff(0.1),
    stretch_saturation(0),

    // merge parameters
    merge_segments(1),
    merge_threshold(35.0),
    minimum_segment_count(2),
    stretch_saturation_merge(0),

    // segment splitting
    connected_segments(1),

    // segment identifier lookup parameters
    segment_adult_age(20),
    segment_age_weight(0.5),
    min_segment_attach_similarity(0.5),

    // delayed segment visualization
    segment_delay(3),

    // saturation stretching function (only used if either
    // stretch_saturation or stretch_saturation_merge is set to 1)
    stretch_point0_x(0.0),   stretch_point0_y(0.0),
    stretch_point1_x(3.0),   stretch_point1_y(1.0),
    stretch_point2_x(20.0),  stretch_point2_y(200.0),
    stretch_point3_x(255.0), stretch_point3_y(255.0),

    // visualization parameters
    visualisation_mode(0),
    drawTrace(0),
    drawCross(1),
    drawLine(1),
    tracelength(10)
{
  return;
}

Setting::Setting(const Setting& clone)
  : // ncut segmentation parameters
    ncut_tree_depth(4),
    number_of_eigenvectors(2),
    number_of_eigenvector_splittest_steps(10),

    // element information gathering parameters
    variation_bins(100),
    saturation_multiplier(4.0),

    // similarity calculation parameters
    intensity_weight(1.0),
    position_weight(0.0),
    intensity_falloff(0.1),
    distance_falloff(0.1),
    stretch_saturation(0),

    // merge parameters
    merge_segments(1),
    merge_threshold(35.0),
    minimum_segment_count(2),
    stretch_saturation_merge(1),

    // segment splitting
    connected_segments(1),

    // segment identifier lookup parameters
    segment_adult_age(20),
    segment_age_weight(0.5),
    min_segment_attach_similarity(0.5),

    // delayed segment visualization
    segment_delay(3),

    // saturation stretching function (only used if either
    // stretch_saturation or stretch_saturation_merge is set to 1)
    stretch_point0_x(0.0),   stretch_point0_y(0.0),
    stretch_point1_x(3.0),   stretch_point1_y(1.0),
    stretch_point2_x(20.0),  stretch_point2_y(200.0),
    stretch_point3_x(255.0), stretch_point3_y(255.0),

    // visualization parameters
    visualisation_mode(0),
    drawTrace(0),
    drawCross(1),
    drawLine(1),
    tracelength(10)
{
  *this = clone;
}

Setting& Setting::operator=(const Setting& clone)
{
  // ncut segmentation paramters
  ncut_tree_depth = clone.ncut_tree_depth;
  number_of_eigenvectors = clone.number_of_eigenvectors;
  number_of_eigenvector_splittest_steps =
      clone.number_of_eigenvector_splittest_steps;

  // element information gathering parameters
  variation_bins = clone.variation_bins;
  saturation_multiplier = clone.saturation_multiplier;

  // similarity calculation parameters
  intensity_weight = clone.intensity_weight;
  position_weight = clone.position_weight;
  intensity_falloff = clone.intensity_falloff;
  distance_falloff = clone.distance_falloff;
  stretch_saturation = clone.stretch_saturation;

  // merge parameters
  merge_segments = clone.merge_segments;
  merge_threshold = clone.merge_threshold;
  minimum_segment_count = clone.minimum_segment_count;
  stretch_saturation_merge = clone.stretch_saturation_merge;

  // segment splitting
  connected_segments = clone.connected_segments;

  // segment identifier lookup parameters
  segment_adult_age = clone.segment_adult_age;
  segment_age_weight = clone.segment_age_weight;
  min_segment_attach_similarity = clone.min_segment_attach_similarity;

  // delayed segment visualization
  segment_delay = clone.segment_delay;

  // saturation stretching function (only used if either
  // stretch_saturation or stretch_saturation_merge is set to 1)
  stretch_point0_x = clone.stretch_point0_x;
  stretch_point0_y = clone.stretch_point0_y;
  stretch_point1_x = clone.stretch_point1_x;
  stretch_point1_y = clone.stretch_point1_y;
  stretch_point2_x = clone.stretch_point2_x;
  stretch_point2_y = clone.stretch_point2_y;
  stretch_point3_x = clone.stretch_point3_x;
  stretch_point3_y = clone.stretch_point3_y;

  // visualization parameters
  visualisation_mode = clone.visualisation_mode;
  drawTrace = clone.drawTrace;
  drawCross = clone.drawCross;
  drawLine = clone.drawLine;
  tracelength = clone.tracelength;

  return *this;
}

Setting::~Setting()
{
  return;
}

}
ncut::Setting ncut_Setting;


#endif // NCUTSETTING_H
